function [payload, header_CRC_valid, payload_len_received, CRC_on, CR] = LoRa_decoder(symbols, SF, DE)
%LORA_DECODER decodes the symbols from a LoRa signal
%   The LoRa_decoder decodes a LoRa message based on the symbols with the
%   spreading factor SF and a low datarate mode used (DE = 1 enabled, 
%   DE = 0 disabled. It returns the payload data and the information
%   contained in the header.

% Authors: Holger Arthaber, Harald Eigner
%          TU Wien (EMCE), www.emce.tuwien.ac.at

% This work is licensed under a Creative Commons Attribution 4.0 International 
% (CC BY 4.0), https://creativecommons.org/licenses/by/4.0/
% You are free to:
%  - Share — copy and redistribute the material in any medium or format
%  - Adapt — remix, transform, and build upon the material for any purpose, 
%    even commercially. 
% Under the following terms:
%  - Attribution — You must give appropriate credit, provide a link to the license, 
%    and indicate if changes were made. You may do so in any reasonable manner, but not 
%    in any way that suggests the licensor endorses you or your use. 


%% Header Decoding

header_sym = round(symbols(1:8)/ 4);     %First 8 symbols
header_bin = zeros(8, SF - 2);    %Header symbols binary
header_gray = zeros(8, SF - 2);   %Header Gray-demapped binary

for idx=1:8
    header_bin(idx, :) = dec2binvec(header_sym(idx), SF - 2);  %Transformation of symbols to bits
    header_gray(idx, :) = bin2gray(header_bin(idx, :));  %Gray-demapping
end

header_deinterleaved = deinterleave(header_gray);     %Header deinterleaved binary
[header_decoded_bin] = decoder(header_deinterleaved, 8);  %Decoding

if SF > 7
    payload_FEC_bin = header_decoded_bin(6:end, :);     %Payload data from first 8 symbols
    header_decoded_bin = header_decoded_bin(1:5, :);    %Decoded header binary
end

[header_CRC_valid, CRC_on, CR, payload_len_received] = decode_header(header_decoded_bin);     %Extract header information


%% Payload Decoding

if size(symbols, 2) > 8  
    payload_sym = symbols(9:end);       %Payload symbols
    payload_sym_len = length(payload_sym);
    payload_bin = zeros(payload_sym_len, SF - 2 * DE);
    payload_gray = zeros(payload_sym_len, SF - 2 * DE);
    blocks = payload_sym_len / CR;
    payload_deinterleaved(1:blocks * SF, 1:CR) = zeros(blocks * SF, CR); 
    
    if DE == 1
        payload_sym=floor(payload_sym / 4);       %Reduced symbolalphabet if low-datarate is active
    end
          
    for idx = 1:payload_sym_len
        payload_bin(idx, :) = dec2binvec(payload_sym(idx), SF - 2 * DE);
        payload_gray(idx, :) = bin2gray(payload_bin(idx, :));
    end
    
    for idx = 1 : blocks
        SF_= (1 : SF - 2 * DE) + (idx - 1)*(SF - 2 * DE);
        CR_= (1 : CR) + (idx - 1) * CR;
        payload_deinterleaved(SF_, 1:CR) = deinterleave(payload_gray(CR_, 1:(SF - 2 * DE)));    %Deinterleaving
    end

    [payload_decoded_bin]=decoder(payload_deinterleaved, CR);   %Decoding

    %Add payload data from first eight symbols
    if SF > 7
        payload_whitened_bin = [payload_FEC_bin' payload_decoded_bin']';
    else
        payload_whitened_bin = payload_decoded_bin;
    end
else
    payload_whitened_bin = payload_FEC_bin;
end

payload_whitened_bin2 = reshape(payload_whitened_bin(1:payload_len_received * 2 + 4 * CRC_on, :)', 8, payload_len_received + 2 * CRC_on)';
payload_dec = bi2de(payload_whitened_bin2);

payload_dewhitened = compute_whitening(payload_dec)';   %Dewhitening

%Check validity of CRC
if CRC_on
    payload = payload_dewhitened(1:end - 2);
    CRC_cal = dec2hex(payload_CRC(payload), 4);
    CRC_cal = flipud(reshape(CRC_cal', 2, 2)');
    CRC_received = dec2hex(payload_dec(end - 1:end), 2);
    if CRC_cal ~= CRC_received
        error('invalid CRC');
    end
else
    payload = payload_dewhitened;
end

end
















